/*
 * Author: Evan Peterson
 * Cal Poly, CPE-471, Winter 2014
 */

#include <math.h>

#include "board.h"

Board::Board(void)
{
   this->board = new Model(CUBE_MESH);
   this->blob = new Model(SPHERE_MESH);
   this->curTrans = glm::vec3(0, -.1, -3);
   this->spinRot = this->endSpinRot = this->leanRot = 0;
}

glm::vec3 Board::getPos(void)
{
   return this->curTrans;
}

void Board::draw(void)
{
   const float maxSpin = 80;
   const float maxLean = 15;
   const float inputScale = .1;
   const float spinInc = 1.5;
   const float xMaxSpeed = 3;
   const float xBound = 10;
   float xSpeed;

   if (g_doBoost && g_boostPoints > 0) {
      g_boardAccel += g_boardAccelRate;
      g_boardAccel = fmin(g_boardAccel, g_boardAccelMax);
      g_boostPoints--;
   }
   else {
      g_boardAccel -= g_boardAccelRate / 2;
      g_boardAccel = fmax(g_boardAccel, g_boardAccelMin);
   }

   g_boardVel += g_boardAccel;
   g_boardVel = fmax(g_boardVelMin, fmin(g_boardVel, g_boardVelMax));

   // Bind model material.
   setMaterial(BOARD_MAT); // Bind model texture.
   glBindTexture(GL_TEXTURE_2D, BLANK_TEX);

   g_mStack.pushMatrix();
      this->leanRot = maxLean * -g_axisValue;
      this->endSpinRot = maxSpin * -g_axisValue;

      // Gradually rotate the board toward a desired position.
      if (this->spinRot < this->endSpinRot)
         this->spinRot += spinInc; else if(this->spinRot > this->endSpinRot)
         this->spinRot -= spinInc;

      // Move the board based on the angle of the "carve".
      xSpeed = xMaxSpeed * sin(DEG_TO_RAD(this->spinRot));
      this->curTrans.x += -inputScale * xSpeed * (g_boardVelMin + g_boardVel);

      // Keep the board from going way off-screen.
      if (this->curTrans.x > xBound)
         this->curTrans.x = xBound;
      else if (this->curTrans.x < -xBound)
         this->curTrans.x = -xBound;

      g_mStack.translate(this->curTrans);
      g_mStack.rotate(this->spinRot, jHat);
      g_mStack.rotate(this->leanRot, kHat);
      // TODO Scale this better. UPDATE Eh, let's leave it for legacy.
      g_mStack.scale(1, .1, 3);

      this->board->draw();

      // Drawing the penguin rider. Magic numbers abound.
      // Well, the penguin IS magic numbers.
      // Deal with it.
      g_mStack.pushMatrix();
         // Bind model texture.
         glBindTexture(GL_TEXTURE_2D, BLANK_TEX);

         // Rescale the thing to uniform proportions.
         g_mStack.scale(1, 10, .333f);
         // On top of the board.
         g_mStack.translate(glm::vec3(0, .15, 0));

         // Draw those feet!
         setMaterial(ORANGE_MAT);
         g_mStack.pushMatrix();
            g_mStack.scale(.2, .05f, .1);
            g_mStack.translate(glm::vec3(.5, -1.5, 2.5));
            this->blob->draw();
         g_mStack.popMatrix();

         g_mStack.pushMatrix();
            g_mStack.scale(.2, .05f, .1);
            g_mStack.translate(glm::vec3(.5, -1.5, -2.5));
            this->blob->draw();
         g_mStack.popMatrix();

         // Draw that torso!
         setMaterial(TUX_MAT);
         g_mStack.rotate(leanRot * 2, kHat);
         g_mStack.pushMatrix();
            g_mStack.scale(.25, .5, .25);
            g_mStack.translate(glm::vec3(0, 1.2, 0));
            //g_mStack.rotate(this->leanRot * 2, kHat);
            g_mStack.rotate(this->spinRot, jHat);
            this->blob->draw();

            // How about the head?!
            g_mStack.pushMatrix();
               g_mStack.scale(1, .5, 1);
               g_mStack.scale(.8, .8, .8);
               g_mStack.translate(glm::vec3(0, 4, 0));
               g_mStack.rotate(30, jHat);
               this->blob->draw();

               // And the beak?? The beak makes the penguin.
               setMaterial(ORANGE_MAT);
               g_mStack.pushMatrix();
                  g_mStack.scale(1.25, .25, .25);
                  g_mStack.translate(glm::vec3(1, 0, 0));
                  this->blob->draw();
               g_mStack.popMatrix();

            g_mStack.popMatrix();

            // Re-proportion the body from earlier.
            g_mStack.scale(4, 2, 4);
            setMaterial(TUX_MAT);

            // The right flipper!
            g_mStack.pushMatrix();
               g_mStack.rotate(spinRot, kHat);
               g_mStack.rotate(-30, iHat);

               g_mStack.pushMatrix();
                  g_mStack.scale(.1, .25, .1);
                  g_mStack.translate(glm::vec3(0, 0, 4));
                  this->blob->draw();
               g_mStack.popMatrix();
            g_mStack.popMatrix();

            // The left flipper!
            g_mStack.pushMatrix();
               g_mStack.rotate(-spinRot, kHat);
               g_mStack.rotate(30, iHat);

               g_mStack.pushMatrix();
                  g_mStack.scale(.1, .25, .1);
                  g_mStack.translate(glm::vec3(0, 0, -4));
                  this->blob->draw();
               g_mStack.popMatrix();
            g_mStack.popMatrix(); // Flippers

         g_mStack.popMatrix(); // Torso

      g_mStack.popMatrix(); // Penguin

   g_mStack.popMatrix(); // Board
}
