/*
 * Author: Evan Peterson
 * Cal Poly, CPE-471, Winter 2014
 */

#include "gate.h"

static const int g_zPosMin = -100;
static const int g_zPosBase = -300;
static const int g_xRange = 10;
static const float g_gateWidthMin = 1.5;
static const float g_gateWidthMax = 7;
static float g_gateWidth = g_gateWidthMax;

Gate::Gate(void)
{
   this->pole = new Model(SMOOTH_CYL_MESH);
   this->init();
}

void Gate::init(void)
{
   if (this->hit == false) {
      // Gate is being reset, and wasn't ever hit. The rider missed!
      // Punish them!
      g_gateWidth = std::min(g_gateWidthMax, g_gateWidth + 2);
      g_boostPoints = fmax(0, g_boostPoints - g_boostReward / 2);
   }

   this->hit = false;
   this->width = g_gateWidth;
   this->xPos = (rand() % (g_xRange - (int)g_gateWidth)) // x-location
                + 1.f / (rand() % 10)                    // plus some fraction 
                - (g_xRange / 2);                        // centered on z-axis
   this->curTrans = glm::vec3(this->xPos, 0, g_zPosBase);
}

void Gate::setZ(int index)
{
   this->curTrans.z = g_zPosMin * index;
}

bool Gate::draw(void)
{
   this->curTrans.z += g_boardVel;

   g_mStack.pushMatrix();

      // Bind model material.
      setMaterial(ORANGE_MAT);
      // Bind model texture.
      glBindTexture(GL_TEXTURE_2D, BLANK_TEX);

      // Right pole (based off off left pole's position.
      g_mStack.pushMatrix();
         g_mStack.translate(glm::vec3(this->curTrans.x + this->width,
                                      this->curTrans.y,
                                      this->curTrans.z));
         g_mStack.scale(.1f, 2.f, .1f);
         g_mStack.rotate(-90, iHat); // new basis: (+X, -Z, +Y)
         this->pole->draw();
      g_mStack.popMatrix();

      // Left pole (the reference pole).
      g_mStack.translate(this->curTrans);
      g_mStack.scale(.1f, 2.f, .1f);
      g_mStack.rotate(-90, iHat); // new basis: (+X, -Z, +Y)
      this->pole->draw();

   g_mStack.popMatrix();

   // The model is still on-screen.
   return this->curTrans.z < g_viewBound;
}

void Gate::checkHit(glm::vec3 boardPos)
{
   // Set up a threshold *in front* of the gate entrance.
   // It's gotta be a biggish threshold since the gate moves by quickly.
   if (this->hit == false) {
      if (boardPos.z > this->curTrans.z &&
          boardPos.z <= this->curTrans.z + g_boardVelMax &&
          boardPos.x > this->xPos &&
          boardPos.x < this->xPos + this->width) {

         this->hit = true;
         g_boostPoints = std::min(g_boostPointsMax,
                                  g_boostPoints + g_boostReward);
         g_gateWidth = fmax(g_gateWidthMin, g_gateWidth - .5);
      }
   }
}
