/*
 * Author: Evan Peterson
 * Cal Poly, CPE-471, Winter 2014
 */

#ifndef GEOM_H
#define GEOM_H

#include "main.h"
#include "texture.h"

#include "third_party/MStackHelp.h"

#define RAD_TO_DEG(rad) rad * 180 / M_PI
#define DEG_TO_RAD(deg) deg * M_PI / 180

#define CYLINDER_MESH 0
#define CONE_MESH 1
#define CUBE_MESH 2
#define PLANE_MESH 3
#define GROUND_MESH 4
#define SPHERE_MESH 5
#define SMOOTH_CYL_MESH 6
#define MAX_MESH_INDEX 6

#define BOARD_MAT 0
#define PINE_MAT 1
#define BARK_MAT 2
#define SNOW_MAT 3
#define TUX_MAT 4
#define ORANGE_MAT 5

static const glm::vec3 iHat(1, 0, 0);
static const glm::vec3 jHat(0, 1, 0);
static const glm::vec3 kHat(0, 0, 1);

// Matrix stack utility.
extern RenderingHelper g_mStack;

// Shader attributes.
extern GLint h_aPosition;
extern GLint h_aNormal;
extern GLint h_aTexCoord;

// Shader uniforms.
extern GLint h_uModelMx;
extern GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine;

class Model
{
   private:
      GLuint vbo, ibo, nbo, tbo;
      int polyCount;

   public:
      ~Model(void) {}
      Model(const int meshIndex);
      void draw(void);
};

void initGeom();
void drawGeom();
void setMaterial(int matId);

extern int g_boostPoints;
extern const int g_boostReward, g_boostPointsMax;
extern float g_boardVel;
extern const float g_boardVelMin, g_boardVelMax;
extern float g_boardAccel;
extern const float g_boardAccelMin, g_boardAccelMax, g_boardAccelRate;

#endif
